#include <stdio.h>
#include <stdlib.h>
#include <math.h>

FILE *fp, *gp;

int matrix_size;
double  d[100], v[100][100], a[100][100], a0[100][100];

main(int argc, char **argv)
{
  int i, j, k, dowker[100], nc, id, seif[100][100], signature;
  
  fp = fopen(argv[1], "r");
  gp = fopen(argv[2], "w");
  
  while (fscanf(fp, "%2d%10d", &nc, &id) != EOF)
  {
    for (i=1; i<=nc; ++i) fscanf(fp, "%4d", dowker+i);
    fscanf(fp, "%2d", &matrix_size);
  
    for (i=1; i<=matrix_size; ++i) for (j=1; j<=matrix_size; ++j)
    {
      fscanf(fp, "%6d", seif[i] + j);
      a0[i][j] = (double) seif[i][j];
    }
  
    for (i=1; i<=matrix_size; ++i) for (j=1; j<=matrix_size; ++j)
      a[i][j] = a0[i][j] + a0[j][i];
  
    jacobi();
    
    signature = 0;
    for (i=1; i<=matrix_size; ++i)
    {
      if (d[i] > 0.0) ++signature;
      if (d[i] < 0.0) --signature;
    }
    
    signature = abs(signature);
    
    fprintf(gp, "%2d %8d\n", nc, id);
    for (i=1; i<=nc; ++i) fprintf(gp, "%4d", dowker[i]);
    fprintf(gp, "\n\n");
    
    fprintf(gp, "Seifert matrix:                     Signature:%4d\n",
                 signature);
    for (i=1; i<=matrix_size; ++i)
    {
      for (j=1; j<=matrix_size; ++j) fprintf(gp, "%3d", seif[i][j]);
      fprintf(gp, "\n");
    }
    
    fprintf(gp, "\n\n");
  }
  
  return 0;
  
}  



jacobi()
{
  double sm, c, s, t, h, g, tau, theta, tresh, z[100], b[100];
  int p, q, i, j, n, rot;
  
  n = matrix_size;
  
  for (p=1; p<=n; ++p) for (q=1; q<=n; ++q)
  {
    if (p == q) v[p][q] = 1.0;
    else v[p][q] = 0.0;
  }
  
  for (p=1; p<=n; ++p)
  {
    b[p] = d[p] = a[p][p];
    z[p] = 0;
  }
  
  rot = 0;
  
  for (i=1; i<=50; ++i)
  {
    sm = 0.0;
    
    for (p=1; p<=n-1; ++p) for (q=p+1; q<=n; ++q)
      sm += fabs(a[p][q]);
    
    if (fabs(sm) == 0) return 0;
    
    if (i < 4) tresh = 0.2 * (sm / (n*n));
    else tresh = 0.0;
    
    for (p=1; p<=n-1; ++p) for (q=p+1; q<=n; ++q)
    {
      g = 100 * fabs(a[p][q]);
      
      if (i>4 && fabs(d[p])+g == fabs(d[p]) && fabs(d[q])+g == fabs(d[q]))
        a[p][q] = 0.0;
      else if (fabs(a[p][q]) > tresh)
      {
        h = d[q] - d[p];
	if (fabs(h) + g == fabs(h))
	  t = a[p][q] / h;
	else
	{
	  theta = 0.5 * h / a[p][q];
	  t = 1 / (fabs(theta) + sqrt(1 + theta*theta));
	  if (theta < 0) t *=-1;
	}
	
	c = 1 / sqrt(1 + t*t);
	s = t * c;
	tau = s / (1 + c);
	h = t * a[p][q];
	
	z[p] = z[p] - h;
	z[q] = z[q] + h;
	d[p] = d[p] - h;
	d[q] = d[q] + h;
	
	a[p][q] = 0;
	
	for (j=1; j<=p-1; ++j)
	{
	  g = a[j][p];
	  h = a[j][q];
	  a[j][p] = g - s * (h + g * tau);
	  a[j][q] = h + s * (g - h * tau);
	}
	
	for (j=p+1; j<=q-1; ++j)
	{
	  g = a[p][j];
	  h = a[j][q];
	  a[p][j] = g - s * (h + g * tau);
	  a[j][q] = h + s * (g - h * tau);
	}
        
	for (j=q+1; j<=n; ++j)
	{
	  g = a[p][j];
	  h = a[q][j];
	  a[p][j] = g - s * (h + g * tau);
	  a[q][j] = h + s * (g - h * tau);
	}
	
	for (j=1; j<=n; ++j)
	{
	  g = v[j][p];
	  h = v[j][q];
	  v[j][p] = g - s * (h + g * tau);
	  v[j][q] = h + s * (g - h * tau);
        }
	
	++rot;
      }
    }
    
    for (p=1; p<=n; ++p)
    {
      d[p] = b[p] = b[p] + z[p];
      z[p] = 0;
    }
  }
  
  return 0;
}
      
      
      
      
      
      
      
      
      
